/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.componentmanager;

//---IMPORTS---
import java.util.HashMap;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.core.*;

/**
 * An XMLCache to cut down the cost of XML parsing.
 * This class tries to ensure we only parse a String of XML
 * into an XML packet once.
 *
 * @author  $Author: tdb $
 * @version $Id: XMLCache.java,v 1.3 2003/02/05 16:43:46 tdb Exp $
 */
public class XMLCache extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.3 $";
    
    /**
     * The default period for cleaning out old XMLPackets
     */
    public final int DEFAULT_CLEANUP_PERIOD = 30;
    
//---STATIC METHODS---

    /**
     * Reference to our instance - we're a singleton.
     */
    private static XMLCache _instance;
    
    /**
     * Static initialiser block for constructing the initial
     * instance of this singleton.
     */
    static {
        _instance = new XMLCache();
    }
    
    /**
     * Return a reference to our singleton.
     */
    public static XMLCache getInstance() {
        return _instance;
    }
    
//---CONSTRUCTORS---

    /**
     * A private constructor to start this class up
     */
    private XMLCache() {
        // set the Thread name
        setName("componentmanager.XMLCache");
        // and start it
        start();
    }

//---PUBLIC METHODS---

    /**
     * Main loop for the thread to clean out old XMLPackets. This is
     * done by swapping HashMap's around. This seemed like the most
     * efficient approach, and is infact a documented garbage collection
     * technique :)
     */
    public void run() {
        while(_running) {
            // get cleanup period from configuration
            int cleanupPeriod = 0;
            try {
                cleanupPeriod = Integer.parseInt(ConfigurationProxy.getInstance().getProperty("XMLCache", "XMLCache.cleanupPeriod"));
            } catch (NumberFormatException e) {
                cleanupPeriod = DEFAULT_CLEANUP_PERIOD;
                _logger.write(toString(), Logger.WARNING, "Erronous XMLCache.cleanupPeriod value in configuration using default of " + cleanupPeriod + " seconds");
            } catch (PropertyNotFoundException e) {
                cleanupPeriod = DEFAULT_CLEANUP_PERIOD;
                _logger.write(toString(), Logger.WARNING, "XMLCache.cleanupPeriod value unavailable using default of " + cleanupPeriod + " seconds");
            }
            
            // wait for determined length of time
            try {
                Thread.sleep(cleanupPeriod * 1000);
            } catch (InterruptedException e) {
            }
            
            // do the swap
            synchronized (this) {
                _hashB = _hashA;
                // creating a new HashMap is quicker than clearing the old one
                _hashA = new HashMap();
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Generates an XMLPacket from a given String of XML, checking
     * to see if it already has the XMLPacket cached.
     *
     * @param xml A String of XML data
     * @return An XMLPacket representing the xml parameter
     * @throws InvalidXMLException if the XML does not parse
     */
    public XMLPacket getXMLPacket(String xml) throws InvalidXMLException {
        // Hash A is more likely to contain the result, assuming we have our
        // swap timing about right :)
        if(_hashA.containsKey(xml)) {
            return (XMLPacket) _hashA.get(xml);
        }
        else if(_hashB.containsKey(xml)) {
            return (XMLPacket) _hashB.get(xml);
        }
        else {
            XMLPacket packet = _xmlPacketMaker.createXMLPacket(xml);
            // always add to hash A
            _hashA.put(xml, packet);
            return packet;
        }
    }

//---ATTRIBUTES---

    // think about initial parameters for HashMap's
    
    /**
     * Our primary HashMap
     */
    private HashMap _hashA = new HashMap();
    
    /**
     * Our secondary HashMap
     */
    private HashMap _hashB = new HashMap();

    /**
     * A reference to an XMLPacketMaker which we can use
     * to generate XMLPackets from XML Strings
     */
    private XMLPacketMaker _xmlPacketMaker = new XMLPacketMaker();

    /**
     * We can set this to false to stop the Thread
     */
    private boolean _running = true;

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = null;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();

//---STATIC ATTRIBUTES---

}
