/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.core;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;
import java.io.*;

/**
 * This class is essentially a Configuration factory.
 * This class implements the Configurator IDL and allows
 * other classes in the system ot obtain their Configuration
 *
 * On construction it requires a reference to the RootPOA
 * to allow it to create Configuration objects to be
 * returned.
 *
 * It also relies on the System.properties to set internal values.
 *
 * @author  $Author: tdb $
 * @version $Id: ConfigurationManagerServant.java,v 1.25 2003/02/05 16:43:46 tdb Exp $
 */
class ConfigurationManagerServant extends ConfigurationManagerPOA {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.25 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates a new ConfiguratorServant
     * This class uses the System.properties to set internal values
     */
    ConfigurationManagerServant() {
        // assign some local variables
        _configPath = System.getProperty("uk.org.iscream.cms.server.ConfigurationLocation");
        _systemConfigFile = System.getProperty("uk.org.iscream.cms.server.SystemConfigurationFile");

        // load the system config
        loadSystemConfig();

        // log our status
        _logger.write(toString(), Logger.SYSINIT, "started");
        _logger.write(toString(), Logger.SYSMSG, "configuration location - " + _configPath);
        _logger.write(toString(), Logger.SYSMSG, "system configuration file - " + _systemConfigFile);
    }

//---PUBLIC METHODS---
    
    /**
     * Returns a Configuration object which contains
     * the configuration data requested by the calling
     * object.
     *
     * This method will look in the systemConfig file
     * for an entry for this "source", if there is no
     * entry it returns a refernce to the system
     * config.  If there are any errors in reading the 
     * configuration, it returns null, the caller is 
     * expected to be able to handle this.
     *
     * This method also checks to see if the system.conf
     * file has been updated and reloads its reference if
     * needed.
     *
     * @param source the configuration required
     * @return the Configuration
     */
    public Configuration getConfiguration(String source) {
        _logger.write(toString(), Logger.SYSMSG, "got request for " + source);

        
        // check to see if we need to reload the system config
        // because it has changed
        if (isModified(_systemConfig.getFileList(), _systemConfig.getLastModified())) {
            _logger.write(toString(), Logger.SYSMSG, "system config changed");
            loadSystemConfig();
        }

        // search config for group membership
        // and obain a list of groups by name
        LinkedList nameGroups = getGroupMembership(source);
        // add the hosts individual config to the start of the list
        nameGroups.addFirst(source);
        
        // this list will be used to compile the groupings
        LinkedList groups = new LinkedList();
                     
        // if we are dealing with a Host.<hostname> request, then we also
        // want to look for ip address details, as configuration entries may relate to it
        // if we can't resolve it, we don't look.
        LinkedList ipGroups = null;
        if (source.startsWith("Host.")) {
            // hostname is after Host.
            String hostname = source.substring(5);
            try {
                String ip = "Host." + InetAddress.getByName(hostname).getHostAddress();
                ipGroups = getGroupMembership(ip);
                ipGroups.addFirst(ip);
                // add to our list of groups
                groups.addAll(ipGroups);
            } catch (UnknownHostException e) {
                _logger.write(toString(), Logger.ERROR, "could not resolve hostname - " + hostname);
            }
        }
        
        // add the rest of the groups to the end        
        groups.addAll(nameGroups);

        Iterator i = groups.iterator();
        String fileList = "";
        while (i.hasNext()) {
            String groupName = (String) i.next();
            _logger.write(toString(), Logger.DEBUG, "looking for config entry for - " + groupName);
            // we look for this entry in the systemConfig
            String configFile = _systemConfig.getProperty("config." + groupName);
            // if there is a config entry then
            if (configFile != null) {
                _logger.write(toString(), Logger.DEBUG, "looking for config tree in - " + configFile);
    
                // get the file list of includes etc + the system config
                String groupFileList = null;
                try {
                    groupFileList = getIncludedFiles(configFile, "") + ";";
                } catch (Exception e) {
                    // not sure what to do here
                    // so we just log the error
                    _logger.write(toString(), Logger.ERROR, "ERROR - " + e);
                }
                if (groupFileList != null) {
                    fileList += groupFileList;
                }
            } else {
                _logger.write(toString(), Logger.DEBUG, "no config entry for - " + groupName);
            }
        }
        // add the system config as the final check
        fileList = _systemConfigFile + ";" + fileList;
        _logger.write(toString(), Logger.DEBUG, "config tree - " + fileList);
        
        // build the configuration
        Configuration config = buildConfiguration(fileList);
        
        // if this is null at this point, then there will have been an error
        return config;
    }
    
    
    /**
     * When passed a file list and a current value for the lastModified
     * of the current configuration, this method compares the value
     * to the actual value of the configuration files to determine 
     * whether or not the configuration has been modified.
     *
     * @param fileList a list of files that the caller uses for configuration
     * @param lastModified the last modified date of the callers configuration
     *
     * @return whether or not the configuration has been modified
     */
    public boolean isModified(String fileList, long lastModified) {
        StringTokenizer st = new StringTokenizer(fileList, ";");
        long newLastModified;
        File currentFile;
        while (st.hasMoreTokens()) {
            currentFile = new File(_configPath, st.nextToken());
            newLastModified = currentFile.lastModified();
            if (newLastModified > lastModified) {
                return true;
            }
        }
        return false;
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    /**
     * This is a recursive function private to this class.
     * It constructs a hierarchy of files as a ";" serperated
     * string which can be used to read in the configuration.
     * This function calls itself.
     * 
     * @param currentFile the current file to be processed
     * @param readFiles used for recursion purposes only, these are the files it has read so far 
     *
     * @return the current list that has been constructed
     *
     * @throws IOException if there is trouble reading the file
     * @throws FileNotFoundException is there is trouble finding the file
     * @throws CircularIncludeException this is if a circular include is detected
     */
    private String getIncludedFiles(String currentFile, String readFiles) throws IOException, FileNotFoundException, Exception {
        
        // check for circular include here
        if (hasDuplicate(currentFile, readFiles) || currentFile.equals(_systemConfigFile)) {
            throw new CircularIncludeException(currentFile + " is included more than once");
        }
        
        // if there wasn't, we're gonna use this file, so make a note of it as read
        // (note the use of the ";", this is for the hasDuplicate, function)
        readFiles = readFiles + currentFile + ";";

        Properties properties = new Properties();
        properties.load(new FileInputStream(new File(_configPath, currentFile)));
        
        // get the include property
        String includes = properties.getProperty("include");

        // if we're the last file with no includes, return our name
        if (includes == null) {
            return currentFile;
        
        // otherwise, recurse over our includes
        } else {
            StringTokenizer st = new StringTokenizer(includes, ";");
            String returnList= "";
            while (st.hasMoreTokens()) {
                returnList = getIncludedFiles(st.nextToken(), readFiles) + ";" + returnList;
            }
            
            return returnList + currentFile;
        }
    }

    /**
     * This simple method checks to see if a given 
     * file exists in the given list.
     *
     * @param file the file to check the list for
     * @param fileList the list to check
     *
     * @return if the given file appeard in the list
     */
    private boolean hasDuplicate(String file, String fileList) {
        StringTokenizer st = new StringTokenizer(fileList, ";");
        while (st.hasMoreTokens()) {
            if (file.equals(st.nextToken())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Opens and loads the system configuration into the 
     * local reference _systemConfig
     */
    private void loadSystemConfig() {
        _logger.write(this.toString(), Logger.SYSMSG, "reloading " + _systemConfigFile);
        // get a reference to the system config and store it
        try {
            // create the properties for the configuration
            File systemConfigFile = new File(_configPath, _systemConfigFile);
            _systemConfigHolder = new Properties();
            _systemConfigHolder.load(new FileInputStream(systemConfigFile));
            
            // create the servant
            ConfigurationServant ref = new ConfigurationServant(_systemConfigHolder, _systemConfigFile, systemConfigFile.lastModified());
            org.omg.CORBA.Object objRef = _refman.getRootPOA().servant_to_reference(ref);
            
            // narrow it to a Configuration
            _systemConfig = ConfigurationHelper.narrow(objRef);
            
        } catch (Exception e) {
            _logger.write(toString(), Logger.FATAL, "ERROR: " + e.getMessage());
        }
    }

    /**
     * Parses the system configuration file
     * for group membership entries.
     *
     * It looks for all entries of group.<name>
     * which contain the given source name
     *
     * @param source the source to find membership for
     *
     * @return the list of groups that this source is a member of
     */
    private LinkedList getGroupMembership(String source) {
        _logger.write(toString(), Logger.DEBUG, "searching groups for - " + source);
        LinkedList groupMembership = new LinkedList();        
        Iterator i = new TreeSet(_systemConfigHolder.keySet()).iterator();
        while(i.hasNext()) {
            String key = (String) i.next();
            // look for a key that's a group entry
            if (key.startsWith("group.")) {
                // get the list of hosts in the group
                String group = _systemConfig.getProperty(key);
                if(groupMatch(source, group)) {
                    groupMembership.add(key.substring(6));
                    _logger.write(toString(), Logger.DEBUG, "group match found for - " + source + " in group - " + key);
                }
            }   
        }
        return groupMembership;
    }
    
    /**
     * Checks that a given source is matched within the
     * given list of hosts. For example:<br>
     * <br>
     * Given "stue5de.ukc.ac.uk"<br>
     * And   "raptor.ukc.ac.uk;stue*.ukc.ac.uk<br>
     * <br>
     * This method would return true as there is a match.
     *
     * This method will also match if the source is exactly
     * matched within the group of hosts (ie. no wildcard).
     *
     * @param source the string to look for
     * @param group the group to search for a match
     *
     * @return if there is a match
     */
    public static boolean groupMatch(String source, String group) {
        StringTokenizer st = new StringTokenizer(group, ";");
        // go through all the hosts in the group
        while (st.hasMoreTokens()) {
            String host = st.nextToken();
            if(StringUtils.wildcardMatch(source, host)) {
                return true;
            }
        }
        // not had a match
        return false;
    }

    /**
     * Build the properties as a Configuration to be 
     * returned to the caller
     *
     * @param fileList the list of files to build the configuration from
     *
     * @return the built Configuration
     */
    private Configuration buildConfiguration(String fileList) {
        Configuration config = null;

        // if there is an entry
        if (!fileList.equals("")) {
            try {
                
                // build the properites here from the filelist.... 
                StringTokenizer st = new StringTokenizer(fileList, ";");
                
                // some holders for variables
                File currentFile;
                long lastModified = 0, newLastModified = 0;
                Properties properties = null, prevProperties = null;
                
                // the root of all configurations will be the system config
                // so we need to open the properties of that
                Properties defaultProperties = new Properties();
                
                // This loop then iterates over the file list
                // creates the properties to be passed to the
                // Configuration constructor
                while (st.hasMoreTokens()) {
                    properties = new Properties(defaultProperties);
                    currentFile = new File(_configPath, st.nextToken());
                    newLastModified = currentFile.lastModified();
                    if (newLastModified > lastModified) {
                        lastModified = newLastModified;
                    }
                    properties.load(new FileInputStream(currentFile));
                    defaultProperties = properties;
                }

                // this creates the configuration, all nice, ready to be returned
                ConfigurationServant ref = new ConfigurationServant(properties, fileList, lastModified);
                org.omg.CORBA.Object objRef = _refman.getRootPOA().servant_to_reference(ref);
                config = ConfigurationHelper.narrow(objRef);
                _logger.write(toString(), Logger.DEBUG, "returning built configuration");
            } catch (Exception e) {
                // not sure what to do here
                // so we just log the error
                _logger.write(toString(), Logger.ERROR, "ERROR - " + e);
            }
            
        // if there isn't an entry for the requested config
        } else {
            _logger.write(toString(), Logger.DEBUG, "no configured config, returning " + _systemConfigFile);
            config = _systemConfig;
        }
        return config;
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = Core.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * The root path to all configurations
     */
    private String _configPath;
    
    /**
     * The name of the file that contains the system configuration
     */
    private String _systemConfigFile;
    
    /**
     * An instance of the system config
     */
    private Configuration _systemConfig;
    
    /**
     * The system config file represented by a
     * properties object.
     */
    private Properties _systemConfigHolder;
    
//---STATIC ATTRIBUTES---
    
}
