/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.core;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import java.util.Properties;
import java.util.Date;
import java.text.DateFormat;

/**
 * An implementation of the Configuration IDL
 * This class allows i-scream modules to read and even
 * set their configuration from a central location.
 *
 * When classes want their configuration, they contact
 * the ConfigurationManager, which will locate their config,
 * open it and pass it to a Configuration object which is
 * then passed back to the calling class.
 *
 * Essentially this class behaves in a similar fashion
 * to the java.util.Properties class.
 *
 * @author  $Author: tdb $
 * @version $Id: ConfigurationServant.java,v 1.23 2003/02/05 16:43:47 tdb Exp $
 */
class ConfigurationServant extends ConfigurationPOA {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.23 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates a new ConfigurationServant taking a hook
     * to a Properties object containing the configuration.
     *
     * @param properties a Properties object that contains the full properties for this configuration
     * @param fileList the list of config files used to build this configuration
     * @param lastModified the most recent last modified value for the file list
     * @param logRef a reference to the logger system
     */
    ConfigurationServant(Properties properties, String fileList, long lastModified) {
        // assign local variables
        _properties = properties;
        _lastModified = lastModified;
        _fileList = fileList;
        _logger.write(toString(), Logger.SYSINIT, "created");
        String date = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM).format(new Date(getLastModified()));
        _logger.write(toString(), Logger.SYSMSG, "last modified - " + date);
        _logger.write(toString(), Logger.DEBUG, "file list - " + _fileList);
    }
 
//---PUBLIC METHODS---

    /**
     * A wrapper for java.util.Properties.getProperty
     * When given a key it returns the value of that key
     * ie, key = value
     *
     * @param key the key the value of which is wanted
     */
    public String getProperty(String key) {
        return _properties.getProperty(key); 
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }
    
    /**
     * Unhooks this Configuration object from the ORB
     */
    public void disconnect() {
        try {
            byte[] oid = _refman.getRootPOA().servant_to_id(this);
            _refman.getRootPOA().deactivate_object(oid);
        } catch(Exception e) {
            _logger.write(this.toString(), Logger.ERROR, "disconnect failed: "+e);
        }
    }
        

//---PRIVATE METHODS---

    /**
     * Overridden for debugging purposes
     * to see when an instance of this class
     * is destroyed
     */
    protected void finalize() throws Throwable {
        _logger.write(this.toString(), Logger.DEBUG, "finalized");
    }

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Returns the date stamp of the configuration file
     * this object is using.
     *
     * @return the last modified time for the file
     */
    public long getLastModified() {
        return _lastModified;
    }

    /**
     * Returns the list of files used to build the Properties
     * this object is using.
     *
     * @return the list of files
     */
    public String getFileList() {
        return _fileList;
    }

//---ATTRIBUTES---

    /**
     * The properties object that this class provides a CORBA interface to
     */
    private Properties _properties = new Properties();

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = Core.NAME;

    /**
     * The date stamp of the configuration file
     * this object is using
     */
    private long _lastModified;
    
    /**
     * The list of files that were used to build this configuration
     */
    private String _fileList;
    
//---STATIC ATTRIBUTES---

}
